/*
 *----------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * WebSphere Commerce
 *
 * (c) Copyright IBM Corp. 2014
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-----------------------------------------------------------------------
 *
 *-----------------------------------------------------------------------
 * The sample contained herein is provided to you "AS IS."
 *
 * It is furnished by IBM as a simple example and has not been thoroughly 
 * tested under all conditions.  IBM, therefore, cannot guarantee its
 * reliability, serviceability or functionality.
 *
 * This sample may include the names of individuals, companies, brands
 * and products in order to illustrate concepts as completely as
 * possible.  All of these names are fictitious and any similarity to the 
 * names and addresses used by actual persons or business enterprises is 
 * entirely coincidental.
 *---------------------------------------------------------------------
 */

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

/**
 * A promotion identifier generator for the purpose of data-loading promotion folders to the FOLDER table.
 * The promotion identifiers are promotion folders (i.e. FOLDER.TYPE="IBM_PromotionFolder") 
 * which corresponds to a date stamp in one of the three following formats:
 * 	1) YYYY
 * 	2) YYYY_MM
 * 	3) YYYY_MM_DD
 * 
 * Promotion folder names, by default, are based on Monday's date for every month 
 * starting from the start year as indicated by the input parameter <start_year>
 * all the way up until the current calendar year.  This serves as the lowest level
 * in the folder hierarchical structure with the format "YYYY_MM_DD".
 * 
 * The middle layer corresponds to the 12 months of each year in the format of "YYYY_MM".
 * For each year, 12 of those folder identifiers are generated. 
 * 
 * The top layer corresponds to the years being looked at in the format of "YYYY".
 * 
 * Usage: PromotionFolders <output_file_path> <start_year>
 * 
 * 	where 	<output_file>	is the absolute file path (i.e. including the file name and its 
 * 							extension) to the output file
 * 			<start_year> 	is the starting year in the form of "YYYY" and 
 * 							it should be less than or equal to the current year;
 * 							It is optional and its default value is set to 2012.
 * 
 * For example: PromotionFolders C:\promotion_folders.csv 2007
 * 
 * Below is an snippet of a sample output:
 * 
 * 	Identifier,ParentFolderIdentifier
 * 	2014,,
 * 	2014_01,2014
 * 	2014_01_06,2014_01
 * 	2014_01_13,2014_01
 * 	2014_01_20,2014_01
 * 	2014_01_27,2014_01
 */
public class PromotionFolders {
	
	// CSV File Column Header
	private static final String DATALOAD_HEADER = "Identifier,ParentFolderIdentifier";
	
	// Date Formatters
	private static final SimpleDateFormat DATE_FORMAT 	= new SimpleDateFormat("yyyy_MM_dd");
	private static final SimpleDateFormat MONTH_FORMAT 	= new SimpleDateFormat("yyyy_MM");
	private static final SimpleDateFormat YEAR_FORMAT 	= new SimpleDateFormat("yyyy");
	
	private static final int DAYS_IN_A_WEEK 	= 7;
	private static final int MONTHS_IN_A_YEAR 	= 12;
	
	private int cur_year; // Current calendar year
	private int str_year; // Starting calendar year
	
	private File outputFile; // Absolute output file path including the file name
	
	// A hash map containing child-parent folder relationship in the form of <child_folder_identifier, parent_folder_identifier>
	private HashMap<String, String> folder_relations;
	
	/**
	 * The main method for the program.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		
		PromotionFolders promoFoldering = new PromotionFolders();
		Calendar cal = Calendar.getInstance();
		promoFoldering.init(cal, args);
		
		promoFoldering.populateFolder(cal); 	// Generate the parent-child folder relations
		// promoFoldering.printFolder();			// Optional: Print the generated relations to the standard output
		promoFoldering.toFile();							// Write the relations to a pre-specified CSV file
	}
	
	/**
	 * Populate the parent-child folder identifiers and their corresponding relationships.
	 *  
	 * @param c an instance of the Calendar class
	 */
	public void populateFolder(Calendar c) {
		// Loops through all the years and for each month in the year populate all the Monday's date
		while (str_year <= c.get(Calendar.YEAR) && c.get(Calendar.YEAR) <= cur_year) {
			String yyyy = YEAR_FORMAT.format(c.getTime());
			String yyyy_mm = MONTH_FORMAT.format(c.getTime());
			String yyyy_mm_dd = DATE_FORMAT.format(c.getTime());
			
			// As long as the current folder has not been looked at, add it to the parent-child folder relationship hash map
			if (!folder_relations.containsKey(yyyy)) {
				folder_relations.put(yyyy, null);
			}
			if (!folder_relations.containsKey(yyyy_mm)) {
				folder_relations.put(yyyy_mm, yyyy);
			}
			if (!folder_relations.containsKey(yyyy_mm_dd)) {
				folder_relations.put(yyyy_mm_dd, yyyy_mm);
			}
			c.add(Calendar.DAY_OF_MONTH, DAYS_IN_A_WEEK);
		}
	}
	
	/**
	 * Print the content of generated CSV file, containing parent-child folder relations to the standard output.
	 */
	public void printFolder() {
		
		if (folder_relations != null && 
				!folder_relations.isEmpty()) {
			System.out.println (DATALOAD_HEADER);
			
			Object[] folder_keys = folder_relations.keySet().toArray();
			Arrays.sort(folder_keys);
			
			for (Object fkey : folder_keys) {
				String parent_folder = folder_relations.get(fkey);
				parent_folder = (parent_folder == null || parent_folder.equalsIgnoreCase("null")) ? "," : parent_folder;
				System.out.println (fkey + "," + parent_folder);
			}
		}
	}
	

	/**
	 * Write the output to a predefined CSV file path.
	 */
	public void toFile() {
		
		final String CARRIAGE_RETURN = "\n";
		
		StringBuffer content = new StringBuffer();
		content.append(DATALOAD_HEADER + CARRIAGE_RETURN);
		
		FileWriter fw = null;
		BufferedWriter bw = null;
		
		try {
			
			// Create the output file on system if it doesn't already exists in the file path specified
			if (outputFile != null && !outputFile.exists()) {
				outputFile.createNewFile();
			}
			
			if (folder_relations != null && 
					!folder_relations.isEmpty()) {
				
				Object[] folder_keys = folder_relations.keySet().toArray();
				Arrays.sort(folder_keys);  // Sort by folder identifiers
				
				// Loop through the parent-child folder relationship hash map and print out its content
				for (Object fkey : folder_keys) {
					String parent_folder = folder_relations.get(fkey);
					parent_folder = (parent_folder == null || parent_folder.equalsIgnoreCase("null")) ? "," : parent_folder;
					content.append(fkey + "," + parent_folder + CARRIAGE_RETURN);
				}
				
				// Write the result to a CSV file given its file path
				fw = new FileWriter(outputFile.getAbsoluteFile());
				bw = new BufferedWriter(fw);
				bw.write(content.toString());
			}
			
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (bw != null) {
					bw.close();
				}
				if (fw != null) {
					fw.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Initialize the calendar.
	 * 
	 * @param c An instance of the Calendar.
	 */
	private void init(Calendar c, String[] args) {
		
		folder_relations = new HashMap<String, String>();
		
		if (args.length < 1 || args.length > 2) {
			printCommandUsage();
			System.exit(1);
		}
		
		try {
			outputFile = new File(args[0]);
			
			// Get the starting year; default value is "2012"
			if (args.length == 2) {
				str_year = Integer.valueOf(args[1]);
			} else {
				str_year = Integer.valueOf("2012");
			}
		} catch (NumberFormatException e) {
			System.out.println ("Unable to convert the specified year \""+str_year+"\" to an integer value.");
			e.printStackTrace();
			System.exit(1);
		} catch (NullPointerException e) {
			System.out.println ("Unable to create the CSV file by its absolute path: "+args[0]);
			System.exit(1);
		}
		
		outputFile = new File(args[0]);
		
		// Set the current year
		c.setTime(new Date());
		cur_year = c.get(Calendar.YEAR);
		c.clear();
		
		// Initialize the date to the first Monday in the month of January in the starting year
		c.set(Calendar.YEAR, str_year);
		c.set(Calendar.MONTH, Calendar.JANUARY);
		c.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
	}
	
	/**
	 * Print the command usage to system out.
	 */
	private void printCommandUsage() {
		System.out.println ("Usage: PromotionFolders <output_file_path> <start_year>");
		System.out.println ("where");
		System.out.println ("\t<output_file_path> is the absolute file path (i.e. including the file name and its extension) to the output file");
		System.out.println ("\t<start_year> is the starting year in the form of \"YYYY\" and it should be less than or equal to the current year; it is an optional parameter with default value set to \"2012\"");
		System.out.println ();
	}
}
