/*
 *----------------------------------------------------------------------
 * Licensed Materials - Property of IBM
 *
 * WebSphere Commerce
 *
 * (c) Copyright IBM Corp. 2014
 *
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *-----------------------------------------------------------------------
 *
 *-----------------------------------------------------------------------
 * The sample contained herein is provided to you "AS IS."
 *
 * It is furnished by IBM as a simple example and has not been thoroughly 
 * tested under all conditions.  IBM, therefore, cannot guarantee its
 * reliability, serviceability or functionality.
 *
 * This sample may include the names of individuals, companies, brands
 * and products in order to illustrate concepts as completely as
 * possible.  All of these names are fictitious and any similarity to the 
 * names and addresses used by actual persons or business enterprises is 
 * entirely coincidental.
 *---------------------------------------------------------------------
 */

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Vector;

/**
 * A sample program for sorting promotions under a specified store into their respective 
 * folders by their start date.  Using Mondays as the starting date of each week, with 
 * each folder name corresponds to a Monday's date on the calendar in the format of "YYYY_MM_DD", 
 * if a promotion's start date falls under that week, such promotion will be sorted under 
 * the folder by placing an entry in the FOLDERITEM table.
 * 
 * For example, if a promotion's start date is defined to be 2014_01_10, then
 * it would be placed under the folder identifier "2014_01_09".
 * 
 * 	Usage: SortPromotionsIntoFolders <folder_store_identifier> <promo_store_identifier> <output_file_path> <dbType> <dbUser> <dbPassword>
 * 	where
 * 		<folder_store_identifier> is the unique store identifier for which the folders are owned by (i.e. Aurora)
 * 		<promo_store_identifier> is the unique store identifier for which the promotions are owned by (i.e. Aurora)
 * 		<output_file> is the absolute file path (i.e. including the file name and its extension) to the output file
 * 		<dbType> is the data base type and can only be one of the following values: <Derby|DB2|Oracle>
 * 		<dbUser> is the data base user name; it is optional for the derby database type
 * 		<dbPassword> is the data base password; it is optional for the derby database type
 * 
 * For example: AuroraStorefrontAssetStore AuroraStorefrontAssetStore C:\promotion_folderitems.csv db2 db2user db2password
 * 
 * Below is an snippet of a sample output:
 * 
 * 	PromotionId,FolderName,PromoStoreIdentifier
 * 	725827884,2003_12_29,AuroraStorefrontAssetStore
 * 	725827885,2003_12_29,AuroraStorefrontAssetStore
 * 	725827886,2003_12_29,AuroraStorefrontAssetStore
 * 	725827887,2003_12_29,AuroraStorefrontAssetStore
 * 	725827888,2003_12_29,AuroraStorefrontAssetStore
 * 	725827889,2003_12_29,AuroraStorefrontAssetStore
 * 	725827890,2003_12_29,AuroraStorefrontAssetStore
 */
public class SortPromotionsIntoFolders {
	
	// Database Types
	private static final String DERBY_DB_TYPE 		= "DERBY";
	private static final String DB2_DB_TYPE 		= "DB2";
	private static final String ORACLE_DB_TYPE 		= "ORACLE";
	
	/*
	 * DB2 Database Connection Configuration
	 */
	private static final String DB2_DRIVER 		= "com.ibm.db2.jcc.DB2Driver";
	private static final String DB2_URL 		= "jdbc:db2://localhost:50000/mall";
	
	/*
	 * ORACLE Database Connection Configuration
	 */
	private static final String ORACLE_DRIVER 	= "oracle.jdbc.driver.OracleDriver";
	private static final String ORACLE_URL 		= "jdbc:oracle:thin:@localhost:1521:orcl";
	
	/*
	 * Derby Database Connection Configuration
	 */
	private static final String DERBY_DRIVER 	= "org.apache.derby.jdbc.EmbeddedDriver";
	private static final String DERBY_URL 		= "jdbc:derby:W:/WCDE_INT70/db/mall";
	
	// CSV File Column Header
	private static final String DATALOAD_HEADER = "PromotionId,FolderName,PromoStoreIdentifier";
	
	// Date Formatters
	private static final SimpleDateFormat DATE_FORMAT 	= new SimpleDateFormat("yyyy_MM_dd");
	private static final SimpleDateFormat MONTH_FORMAT 	= new SimpleDateFormat("yyyy_MM");
	private static final SimpleDateFormat YEAR_FORMAT 	= new SimpleDateFormat("yyyy");
	
	private String promo_store_identifier;	// store identifiers for the promotions
	private String folder_store_identifier;	// store identifiers for the folders
	
	private String dbType;		// database type
	private String dbUser;		// database user
	private String dbPassword;	// database password
	
	// Absolute output file path including the file name
	private File outputFile;
	
	// A vector storing folder identifiers
	private Vector<String> folders;
	
	// A hash map containing promotion-folder relationship in the form of <promotion_id, folder_identifier>
	private HashMap<String, String> promotion_folder_relations;
	
	/**
	 * The main method for the program.
	 * 
	 * @param args
	 */
	public static void main(String[] args) {
		
		SortPromotionsIntoFolders sortPromotions = new SortPromotionsIntoFolders();
		Calendar cal = Calendar.getInstance();
		sortPromotions.init(args);
		
		sortPromotions.sortPromotionsIntoFolders(cal);  // Generate the promotion-folder relationships
		sortPromotions.printFolderItem();				// Optional: Print the generated relations to the standard output
		sortPromotions.toFile();						// Write the relations to a pre-specified CSV file
	}
	
	/**
	 * Populate the promotion-folder relationships based on promotion's start date.
	 * For instance, if a promotion's start date is defined to be 2014_01_10, then
	 * it would be filed under the folder identifier "2014_01_09".
	 * 
	 * @param c an instance of the Calendar class
	 */
	public void sortPromotionsIntoFolders(Calendar c) {
		
		final String IBM_PROMOTIONFOLDER = "IBM_PromotionFolder";
		
		// FOLDER table selective column names
		final String COL_FOLDER_ID	= "FOLDER_ID";
		final String COL_IDENTIFIER = "IDENTIFIER";
		
		// PX_PROMOTION table selective column names
		final String COL_PX_PROMOTION_ID 	= "PX_PROMOTION_ID";
		final String COL_STARTDATE 			= "STARTDATE";
		final String COL_NAME 				= "NAME";
		
		// SQL query for getting all promotion ids from PX_PROMOTION table that are not in folders under a specified store
		final String GET_PROMOTIONS 		= "select P.PX_PROMOTION_ID, P.STARTDATE from PX_PROMOTION P, STOREENT S where P.STOREENT_ID = S.STOREENT_ID " +
												" and S.IDENTIFIER = '" + promo_store_identifier + "'" +
												" and NOT EXISTS " +
												"(select REFERENCE_ID from FOLDERITEM where FOLDERITEMTYPE = 'PromotionType' and STOREENT_ID=P.STOREENT_ID)";
		
		// SQL query for getting all promotion folder identifiers from FOLDER table under a specified store
		final String GET_PROMOTIONFOLDERS 	= "select F.IDENTIFIER from FOLDER F, STOREENT S where F.STOREENT_ID = S.STOREENT_ID and F.TYPE='IBM_PromotionFolder' and S.IDENTIFIER = '"+folder_store_identifier+"'";
		
		Connection dbc 			= null;
		ResultSet rs 			= null;
		PreparedStatement ps 	= null;
		
		try {
			/*
			 * Establish a data base connection and retrieve all promotion folder identifiers 
			 * under a specified store
			 */
			dbc = this.getDBConnection();
			ps = dbc.prepareStatement(GET_PROMOTIONFOLDERS);
			ps.execute();
			ps.clearParameters();
			
			rs = ps.getResultSet();
			
			if (rs != null) {
				while (rs.next()) {
					folders.add(rs.getString(COL_IDENTIFIER));
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				if (ps != null)		{ps.close();}
				if (rs != null) 	{rs.close();}
				if (dbc != null)	{dbc.close();}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		
		try {
			/*
			 * Establish a data base connection and sort all promotions that are not already in folders 
			 * under a specified store into folders 
			 */
			dbc = this.getDBConnection();
			ps = dbc.prepareStatement(GET_PROMOTIONS);
			ps.execute();
			ps.clearParameters();
			
			rs = ps.getResultSet();
			
			if (rs != null) {
				while (rs.next()) {
					// Get the Monday's date of the week for which the promotion's start date falls under
					c.clear();
					c.setTime(rs.getDate(COL_STARTDATE));
					c.set(Calendar.DAY_OF_WEEK, Calendar.MONDAY);
					
					int promo_id 		= rs.getInt(COL_PX_PROMOTION_ID);		// Get the promotion id
					String folder_name	= DATE_FORMAT.format(c.getTime());		// Get the Monday's date of the week based on promotion's starting date
					
					// As long as the folder already exists, add the promotion-folder relationship
					if (folders.contains(folder_name)) {
						promotion_folder_relations.put(String.valueOf(promo_id), folder_name);
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				if (ps != null)		{ps.close();}
				if (rs != null) 	{rs.close();}
				if (dbc != null)	{dbc.close();}
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Print the content of generated CSV file, containing promotion id-folder relations to the standard output.
	 */
	public void printFolderItem() {
		
		if (promotion_folder_relations != null && 
				!promotion_folder_relations.isEmpty()) {
			System.out.println (DATALOAD_HEADER);
			
			Object[] promo_ids = promotion_folder_relations.keySet().toArray();
			Arrays.sort(promo_ids);
			
			for (Object pkey: promo_ids) {
				String folder_name = promotion_folder_relations.get(pkey);
				folder_name = (folder_name == null || folder_name.equalsIgnoreCase("null")) ? "," : folder_name;
				System.out.println (pkey + "," + folder_name + "," + promo_store_identifier);
			}
		}
	}
	
	/**
	 * Write the output to a predefined CSV file path.
	 */
	public void toFile() {
		
		final String CARRIAGE_RETURN = "\n";
		
		StringBuffer content = new StringBuffer();
		content.append(DATALOAD_HEADER + CARRIAGE_RETURN);
		
		FileWriter fw = null;
		BufferedWriter bw = null;
		
		try {
			
			// Create the output file on system if it doesn't already exists in the file path specified
			if (outputFile != null && !outputFile.exists()) {
				outputFile.createNewFile();
			}
			
			if (promotion_folder_relations != null && 
					!promotion_folder_relations.isEmpty()) {
				
				Object[] promo_ids = promotion_folder_relations.keySet().toArray();
				Arrays.sort(promo_ids);
				
				// Loop through the promotion id-folder relationship hash map and print out its content
				for (Object pkey: promo_ids) {
					String folder_name = promotion_folder_relations.get(pkey);
					folder_name = (folder_name == null || folder_name.equalsIgnoreCase("null")) ? "," : folder_name;
					content.append(pkey + "," + folder_name + "," + promo_store_identifier + CARRIAGE_RETURN);
				}
				
				// Write the result to a CSV file given its file path
				fw = new FileWriter(outputFile.getAbsoluteFile());
				bw = new BufferedWriter(fw);
				bw.write(content.toString());
			}
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				if (bw != null) {
					bw.close();
				}
				if (fw != null) {
					fw.close();
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}
	
	/**
	 * Returns a data base connection to the respective data base type configured above.
	 * 
	 * @return a database connection
	 */
	private Connection getDBConnection() {
		Connection db_connection = null;
		
		String dbURL = null;
		String dbDriver = null;
		
		try {
			// Identify the appropriate database driver
			if (dbType.equalsIgnoreCase(DERBY_DB_TYPE)) {
				dbDriver = DERBY_DRIVER;
				dbURL = DERBY_URL;
			} else if (dbType.equalsIgnoreCase(DB2_DB_TYPE)) {
				dbDriver = DB2_DRIVER;
				dbURL = DB2_URL;
			} else if (dbType.equalsIgnoreCase(ORACLE_DB_TYPE)) {
				dbDriver = ORACLE_DRIVER;
				dbURL = ORACLE_URL;
			}
			
			// Attempt to retrieve the database connection
			Class.forName(dbDriver);
			db_connection = DriverManager.getConnection(dbURL, dbUser, dbPassword);
			
			// Unable to establish database connection; automatically exit the program.
			if (db_connection == null) {
				System.out.println ("Unable to establish a data base connection with the following configuration:");
				System.out.println ("Database Type: "+dbType);
				System.out.println ("Database User: "+dbUser);
				System.out.println ("Database User Password: "+dbPassword);
				System.out.println ("Database Driver: "+dbDriver);
				System.out.println ("Database URL: "+dbURL);
				System.exit(1);
			}
		} catch (ClassNotFoundException e) {
			System.out.println ("Database Connection Failed: Unable to find the data base driver class "+dbDriver+" in the class path.");
			e.printStackTrace();
		} catch (SQLException e) {
			System.out.println ("Database Connection Failed: An exception occurred while attempting to establish a database connection.");
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return db_connection;
	}
	
	/**
	 * Validate and initialize the input arguments being passed in by the user.
	 */
	private void init(String[] args) {
		
		folders = new Vector<String>();
		promotion_folder_relations = new HashMap<String, String>();
		
		if (args.length < 4 || args.length > 6) {
			printCommandUsage();
			System.exit(1);
		}
		
		folder_store_identifier = args[0];
		promo_store_identifier 	= args[1];
		String strOutputFile	= args[2];
		dbType 					= args[3];
		
		if (folder_store_identifier == null || folder_store_identifier.equals("") ||
			promo_store_identifier == null || promo_store_identifier.equals("")) {
			printCommandUsage();
			System.out.println ("Need to specify the store identifiers, <folder_store_identifier> and <promo_store_identifier>, for both the folder as well as the promotions respectively (i.e. Aurora)");
			System.exit(1);
		}
			
		if (strOutputFile == null || strOutputFile.equals("")) {
			printCommandUsage();
			System.out.println ("Need to specify the absolute path for the CSV output file (i.e. /opt/outputCSVFile.csv)");
			System.exit(1);
		}
		
		if (!dbType.equalsIgnoreCase(DERBY_DB_TYPE) &&
			!dbType.equalsIgnoreCase(DB2_DB_TYPE) &&
			!dbType.equalsIgnoreCase(ORACLE_DB_TYPE)) {
			printCommandUsage();
			System.out.println ("Need to specify one of the supported database types, <dbType>: Derby, DB2, Oracle");
			System.exit(1);
		}
		
		if ((dbType.equalsIgnoreCase(DB2_DB_TYPE) || dbType.equalsIgnoreCase(ORACLE_DB_TYPE)) &&
			args.length != 6) {
			printCommandUsage();
			System.out.println ("Need to specify the username and password, <dbUser> and <dbPassword> respectively, for the database type "+dbType);
			System.exit(1);
		} else if (args.length == 6) {
			dbUser		= args[4];
			dbPassword	= args[5];
		}
		
		outputFile = new File(strOutputFile);
	}
	
	/**
	 * Print the command usage to system out.
	 */
	private void printCommandUsage() {
		System.out.println ("Usage: SortPromotionsIntoFolders <folder_store_identifier> <promo_store_identifier> <output_file_path> <dbType> <dbUser> <dbPassword>");
		System.out.println ("where");
		System.out.println ("\t<folder_store_identifier> is the unique store identifier for which the folders are owned by (i.e. Aurora)");
		System.out.println ("\t<promo_store_identifier> is the unique store identifier for which the promotions are owned by (i.e. Aurora)");
		System.out.println ("\t<output_file> is the absolute file path (i.e. including the file name and its extension) to the output file");
		System.out.println ("\t<dbType> is the data base type and can only be one of the following values: <Derby|DB2|Oracle>");
		System.out.println ("\t<dbUser> is the data base user name; it is optional for the derby data base type");
		System.out.println ("\t<dbPassword> is the data base password; it is optional for the derby data base type");
		System.out.println ();
	}
}
